package Plugins::digitalSTROM::Plugin;

# digitalSTROM plugin by Manfred Stock.
# Loosely based on a HelloWorld tutorial plugin by Mitch Gerdisch
#
# This code is derived from code with the following copyright message:
#
# SqueezeCenter Copyright 2001-2007 Logitech.
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License,
# version 2.

use strict;
use warnings;

use base qw(Slim::Plugin::Base);

use Plugins::digitalSTROM::Settings;
use Scalar::Util qw(blessed);
use Slim::Control::Request;
use Slim::Utils::Log;
use Slim::Utils::Prefs;
use Slim::Utils::Strings qw(string);
use Slim::Buttons::Common;
use Carp;
use Data::Dumper;
use WebService::dSS;
use URI;

my $prefs = preferences('plugin.digitalstrom');

my $log = Slim::Utils::Log->addLogCategory({
	'category'     => 'plugin.digitalstrom',
	'defaultLevel' => 'INFO',
#	'defaultLevel' => 'DEBUG',
	'description'  => getDisplayName(),
});

=head1 NAME

Plugins::digitalSTROM::Plugin - Perl plugin for digitalSTROM support in the Squeezebox Server.

=head1 DESCRIPTION

Plugins::digitalSTROM::Plugin is a plugin for the Squeezebox Server which
supports digitalSTROM.

=head1 METHODS

=head2 getDisplayName

Getter for the name that should be displayed on the device.

=head3 Result

The name.

=cut

sub getDisplayName {
	return 'PLUGIN_DIGITALSTROM';
}


=head2 initPlugin

Initialize the plugin.

=cut

sub initPlugin {
	my $class = shift;

	$class->SUPER::initPlugin();

	Plugins::digitalSTROM::Settings->new();

	# Set default for DSS URL
	my $dssurl = $prefs->get('dssurl');
	if (! defined $dssurl || $dssurl eq '') {
		$prefs->set('dssurl', "https://localhost:8080/");
	}
}


=head2 setMode

Handle mode changes. Called on entering and leaving the plugin. Displays a list
of available zones.

=cut

sub setMode {
	my ($class, $client, $method) = @_;

	if ($method eq 'pop') {
		Slim::Buttons::Common::popMode($client);
		return;
	}

	# Get available zones from dSS
	my $api = WebService::dSS->new({
		url => URI->new($prefs->get('dssurl')),
	});
	my @zones      = $api->get_apartment()->get_zones();
	my @zone_names = map {
		my $name = $_->get_name();
		if (defined $name && $name ne '') {
			$name;
		}
		else {
			$client->string('PLUGIN_DIGITALSTROM_ZONE').' '.$_->{id};
		}
	} @zones;

	# Add zones to menu
	my $params = {
		listRef         => \@zone_names,
		header          => '{PLUGIN_DIGITALSTROM}',
		headerAddCount  => 1,
		stringHeader    => 1,
		overlayRef      => sub { return [ undef, shift->symbols('rightarrow') ] },
		callback        => sub {
			my ($client, $exittype) = @_;

			$exittype = uc($exittype);

			if ($exittype eq 'LEFT') {
				Slim::Buttons::Common::popModeRight($client);
			}
			elsif ($exittype eq 'RIGHT') {
				my $list_index = $client->modeParam('listIndex');

				$class->showGroupSelectionMenu({
					zone   => $zones[$list_index],
					client => $client,
				});
			}
			else {
				$client->bumpRight();
			}
		},
	};

	Slim::Buttons::Common::pushMode($client, 'INPUT.Choice', $params);
}


=head2 showGroupSelectionMenu

Show a menu with the available groups for a zone.

=head3 Parameters

This method expects its parameters as a hash reference.

=over

=item zone

The zone that should be displayed.

=item client

The client to use.

=back

=cut

sub showGroupSelectionMenu {
	my ($class, $arg_ref) = @_;

	unless (defined $arg_ref && ref $arg_ref eq 'HASH') {
		croak("No parameters passed or not a hash reference");
	}

	my $zone = $arg_ref->{zone};
	unless (blessed $zone && $zone->isa('WebService::dSS::Apartment::Zone')) {
		croak("Required 'zone' parameter not passed or not a 'WebService::dSS::Apartment::Zone' instance");
	}

	my $client = $arg_ref->{client};
	unless (defined $client) {
		croak("Required 'client' parameter not passed");
	}

	my $zone_name = $zone->get_name();
	unless (defined $zone_name && $zone_name ne '') {
		$zone_name = $client->string('PLUGIN_DIGITALSTROM_ZONE').' '.$zone->get_id();
	}

	# Get groups
	my @groups      = $zone->get_groups();
	my @group_names = map {
		my $name = $_->get_name();
		my $id   = $_->get_id();
		(defined $name && $name ne '') ? $name : $id;
	} @groups;

	# Setup menu
	my $params = {
		listRef        => \@group_names,
		headerAddCount => 1,
		header         => $client->string('PLUGIN_DIGITALSTROM').' - '.$zone_name,
		stringHeader   => 1,
		overlayRef     => sub { return [ undef, shift->symbols('rightarrow') ] },
		callback       => sub {
			my ($client, $exittype) = @_;

			$exittype = uc($exittype);

			if ($exittype eq 'LEFT') {
				Slim::Buttons::Common::popModeRight($client);
			}
			elsif ($exittype eq 'RIGHT') {
				my $list_index = $client->modeParam('listIndex');
				$class->showGroupMenu({
					group  => $groups[$list_index],
					client => $client,
				});
			}
			else {
				$client->bumpRight();
			}
		},
	};

	Slim::Buttons::Common::pushMode($client, 'INPUT.Choice', $params);
	$client->update;
}


=head2 showGroupMenu

Show a menu with options for a given group.

=head3 Parameters

This method expects its parameters as a hash reference.

=over

=item group

The group the menu should be shown for.

=item client

The client to use.

=back

=cut

sub showGroupMenu {
	my ($class, $arg_ref) = @_;

	unless (defined $arg_ref && ref $arg_ref eq 'HASH') {
		croak("No parameters passed or not a hash reference");
	}

	my $group = $arg_ref->{group};
	unless (blessed $group && $group->isa('WebService::dSS::Apartment::Zone::Group')) {
		croak("Required 'group' parameter not passed or not a 'WebService::dSS::Apartment::Zone::Group' instance");
	}

	my $client = $arg_ref->{client};
	unless (defined $client && $client ne '') {
		croak("Required 'client' parameter not passed");
	}

	my @values = (
		{
			name  => '{ON}',
			value => 1,
		},
		{
			name  => '{OFF}',
			value => 0
		}
	);

	# Setup menu
	my $params = {
		listRef        => [@values],
		headerAddCount => 1,
		header         => $client->string('PLUGIN_DIGITALSTROM').' - '.$group->get_name(),
		overlayRef     => sub { return [ undef, shift->symbols('rightarrow') ] },
		callback       => sub {
			my ($client, $exittype) = @_;

			$exittype = uc($exittype);

			if ($exittype eq 'LEFT') {
				Slim::Buttons::Common::popModeRight($client);
			}
			elsif ($exittype eq 'RIGHT') {
				my $list_index = $client->modeParam('listIndex');
				my $value = $client->modeParam('listRef')->[$list_index]->{value};

				if ($value == 1) {
					$group->turn_on();
				}
				else {
					$group->turn_off();
				}

				Slim::Buttons::Common::popModeRight($client);
			}
			else {
				$client->bumpRight();
			}
		},
	};

	Slim::Buttons::Common::pushMode($client, 'INPUT.Choice', $params);
	$client->update;
}


# Nothing (yet)

sub getFunctions {
	return{};
}


1;

=head1 COPYRIGHT AND LICENSE

Copyright 2010-2011 Manfred Stock, futureLAB AG.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
